package com.ruoyi.system.service.impl;

import java.util.List;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDictType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.domain.EtAreaRule;
import com.ruoyi.system.domain.SysStudent;
import com.ruoyi.system.mapper.EtAreaRuleMapper;
import com.ruoyi.system.mapper.SysStudentMapper;
import com.ruoyi.system.service.IAsDeviceService;
import com.ruoyi.system.service.IEtFeeRuleService;
import jdk.nashorn.internal.ir.annotations.Reference;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtOperatingAreaMapper;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.service.IEtOperatingAreaService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 运营区Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
@Slf4j
@Service
public class EtOperatingAreaServiceImpl extends ServiceImpl<EtOperatingAreaMapper, EtOperatingArea> implements IEtOperatingAreaService
{
    @Resource
    private EtOperatingAreaMapper dao;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Resource
    private EtAreaRuleMapper etAreaRuleMapper;


    /**
     * 查询运营区
     *
     * @param areaId 运营区主键
     * @return 运营区
     */
    @Override
    public EtOperatingArea selectEtOperatingAreaByAreaId(Long areaId)
    {
        return dao.selectEtOperatingAreaByAreaId(areaId);
    }

    /**
     * 查询运营区列表
     *
     * @param etOperatingArea 运营区
     * @return 运营区
     */
    @Override
    public List<EtOperatingArea> selectEtOperatingAreaList(EtOperatingArea etOperatingArea)
    {
        List<EtOperatingArea> etOperatingAreas = dao.selectEtOperatingAreaList(etOperatingArea);
        for(EtOperatingArea area : etOperatingAreas){
            Integer integer = asDeviceService.selectCountByAreaId(area.getAreaId());
            area.setDeviceNum(integer);
            List<String> list = etFeeRuleService.selectRuleNameListByAreaId(area.getAreaId());
            StringBuilder ruleStr = new StringBuilder();
            for (String rule : list) {
                if (ruleStr.length() > 0) {
                    ruleStr.append(", ");
                }
                ruleStr.append(rule);
            }
            area.setRuleStr(ruleStr.toString().trim());
        }
        return etOperatingAreas;
    }

    /**
     * 新增运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    @Transactional
    @Override
    public int insertEtOperatingArea(EtOperatingArea etOperatingArea)
    {
        etOperatingArea.setCreateTime(DateUtils.getNowDate());
        /** 将边界值转Geometry对象*/
        String boundaryStr = etOperatingArea.getBoundaryStr();
        Geometry geometry = GeoUtils.toGeometry(boundaryStr);
        String wkt = GeoUtils.wkt(geometry);
        etOperatingArea.setBoundary(wkt);
        int insert = dao.insert(etOperatingArea);
        Long[] ruleIds = etOperatingArea.getRuleIds();
        for (Long ruleId:ruleIds){
            etAreaRuleMapper.insert(EtAreaRule.builder().areaId(etOperatingArea.getAreaId()).ruleId(ruleId).build());
        }
        return insert;
    }

    /**
     * 修改运营区
     *
     * @param etOperatingArea 运营区
     * @return 结果
     */
    @Override
    public int updateEtOperatingArea(EtOperatingArea etOperatingArea)
    {
        /** 将边界值转Geometry对象*/
        String boundaryStr = etOperatingArea.getBoundaryStr();
        Geometry geometry = GeoUtils.toGeometry(boundaryStr);
        String wkt = GeoUtils.wkt(geometry);
        etOperatingArea.setBoundary(wkt);
        return dao.updateById(etOperatingArea);
    }

    /**
     * 批量删除运营区
     *
     * @param areaIds 需要删除的运营区主键
     * @return 结果
     */
    @Override
    public int deleteEtOperatingAreaByAreaIds(Long[] areaIds)
    {
        for (Long areaId:areaIds) {
            dao.deleteById(areaId);
        }
        return 1;
    }

    /**
     * 删除运营区信息
     *
     * @param areaId 运营区主键
     * @return 结果
     */
    @Override
    public int deleteEtOperatingAreaByAreaId(Long areaId)
    {
        return dao.deleteById(areaId);
    }

    /**
     * 根据所有运营分区
     *
     * @return 字典类型集合信息
     */
    @Override
    public List<EtOperatingArea> selectAreaAll()
    {
        return dao.selectList(Wrappers.query());
    }

    /**
     * 校验运营区是否唯一
     *
     * @param etOperatingArea 运营区信息
     * @return 结果
     */
    @Override
    public boolean checkAreaNameUnique(EtOperatingArea etOperatingArea)
    {
        Long areaId = StringUtils.isNull(etOperatingArea.getAreaId()) ? -1L : etOperatingArea.getAreaId();
        QueryWrapper<EtOperatingArea> queryWrapper = Wrappers.query();
        queryWrapper.eq("area_name", etOperatingArea.getAreaName());
        EtOperatingArea info = dao.selectOne(queryWrapper);
        if (StringUtils.isNotNull(info) && info.getAreaId().longValue() != areaId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
}
